#ifndef __SPHERE_H__
#define __SPHERE_H__

#ifdef LINUX
	#include <GL/glew.h>
	#include <GL/gl.h>
#endif
#ifdef DARWIN
	#include <OpenGL/gl.h>
#endif

#include <ngl/ShaderManager.h>
#include <ngl/TransformStack.h>
#include <ngl/Vector.h>

/*! \brief a simple sphere class */
class Sphere
{
public :	
	/*! ctor
	\param pos the position of the sphere
	\param rad the radius of the sphere */
	Sphere(
	       ngl::Vector _pos,
	       GLfloat _rad
	      );
	/*! draw method*/
	void draw(
						 const std::string &_shaderName,
						 ngl::TransformStack &_transformStack

					  )const ;

	inline void setHit(){m_hit=true;}
	inline void setNotHit(){m_hit=false;}
	inline bool isHit()const {return m_hit;}
	inline ngl::Vector getPos() const {return m_pos;}
	inline GLfloat getRadius() const {return m_radius;}
private :
	/*! the position of the sphere */
	ngl::Vector m_pos;
	/*! the radius of the sphere */
	GLfloat m_radius;
	/*! flag to indicate if the sphere has been hit by ray */
	bool m_hit;


};





#endif
