#include "Plane.h"
#include <ngl/Matrix.h>
#include <ngl/Util.h>
Plane::Plane(
              ngl::Vector _center,
              GLfloat _w,
              GLfloat _d
            )
{
  //store the values
  m_center=_center;
  m_width=_w;
  m_depth=_d;
  // now create the vertices based on the values passed in
  m_oVerts[0].m_x=m_center.m_x-(m_width/2.0f);
  m_oVerts[0].m_y=m_center.m_y;
  m_oVerts[0].m_z=m_center.m_z+(m_depth/2.0f);

  m_oVerts[1].m_x=m_center.m_x+(m_width/2.0f);
  m_oVerts[1].m_y=m_center.m_y;
  m_oVerts[1].m_z=m_center.m_z+(m_depth/2.0f);

  m_oVerts[2].m_x=m_center.m_x+(m_width/2.0f);
  m_oVerts[2].m_y=m_center.m_y;
  m_oVerts[2].m_z=m_center.m_z-(m_depth/2.0f);


  m_oVerts[3].m_x=m_center.m_x-(m_width/2.0f);
  m_oVerts[3].m_y=m_center.m_y;
  m_oVerts[3].m_z=m_center.m_z-(m_depth/2.0f);
  // finally calculate the surface m_normal
  m_normal=calcNormal(m_oVerts[3],m_oVerts[2],m_oVerts[1]);
  m_xrot=m_zrot=0;
}

Plane::Plane()
{
// default ctor set 0.0 as origin and unit size
m_center.set(0,0,0);
m_width=1.0;
m_depth=1.0;

m_oVerts[0].m_x=m_center.m_x-(m_width/2.0f); m_oVerts[0].m_y=m_center.m_y; m_oVerts[0].m_z=m_center.m_z+(m_depth/2.0f);
m_oVerts[1].m_x=m_center.m_x+(m_width/2.0f); m_oVerts[1].m_y=m_center.m_y; m_oVerts[1].m_z=m_center.m_z+(m_depth/2.0f);
m_oVerts[2].m_x=m_center.m_x+(m_width/2.0f); m_oVerts[2].m_y=m_center.m_y; m_oVerts[2].m_z=m_center.m_z-(m_depth/2.0f);
m_oVerts[3].m_x=m_center.m_x-(m_width/2.0f); m_oVerts[3].m_y=m_center.m_y; m_oVerts[3].m_z=m_center.m_z-(m_depth/2.0f);
m_normal=ngl::calcNormal(m_oVerts[3],m_oVerts[2],m_oVerts[1]);
}

Plane::~Plane()
{
// do nothing for now
}

void Plane::draw(
								const std::string &_shaderName,
                 ngl::TransformStack &_transformStack
							  )
{
	// create the m_points array for drawing the triangel
	std::vector <GLfloat> points;
	std::vector <GLfloat> normals;

	ngl::Vector normal=ngl::calcNormal(m_verts[0],m_verts[2],m_verts[1]);

	for(int i=0; i<4; ++i)
	{
		points.push_back(m_verts[i].openGL()[0]);
		points.push_back(m_verts[i].openGL()[1]);
		points.push_back(m_verts[i].openGL()[2]);
		normals.push_back(normal.openGL()[0]);
		normals.push_back(normal.openGL()[1]);
		normals.push_back(normal.openGL()[2]);
	}

  ngl::ShaderManager *shader=ngl::ShaderManager::instance();
	(*shader)[_shaderName]->use();

	// first we draw the triangle
	// we build up a vertex array for the lines of the start and end points and draw
	_transformStack.pushTransform();
	{
		//shader->setShaderParamFromMatrix(_shaderName,"ModelMatrix",_transformStack.getCurrAndGlobal().getMatrix());
		_transformStack.loadGlobalAndCurrentMatrixToShader(_shaderName,"ModelMatrix");

		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);

		glVertexPointer(3, GL_FLOAT, 0, &points[0]);
		glNormalPointer( GL_FLOAT, 0, &normals[0]);

		glDrawArraysInstancedARB(GL_QUADS, 0, 4,1);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);

	}
	_transformStack.popTransform();


	_transformStack.pushTransform();
	{
		GLfloat lines[2*3];
		lines[0]=m_center.openGL()[0];
		lines[1]=m_center.openGL()[1];
		lines[2]=m_center.openGL()[2];
		lines[3]=m_normal.m_x*4.0;
		lines[4]=m_normal.m_y*4.0;
		lines[5]=m_normal.m_z*4.0;

		//shader->setShaderParamFromMatrix(_shaderName,"ModelMatrix",_transformStack.getCurrAndGlobal().getMatrix());
		_transformStack.loadGlobalAndCurrentMatrixToShader(_shaderName,"ModelMatrix");

		glEnableClientState(GL_VERTEX_ARRAY);

		glVertexPointer(3, GL_FLOAT, 0, &lines[0]);

		glDrawArraysInstancedARB(GL_LINES, 0, 2,1);
		glDisableClientState(GL_VERTEX_ARRAY);

	}
	_transformStack.popTransform();



	/*
// draw the plamne
glPushMatrix();
	glColor3f(1,1,1);
  m_center.Translate();
	glBegin(GL_QUADS);
    m_normal.Normal();
		for(int i=0; i<4; i++)
			{
      m_verts[i].Vertex();
			}
	glEnd();
  // now draw the surface m_normal
	glDisable(GL_LIGHTING);
		glBegin(GL_LINES);
			glColor3f(1,1,1);
      m_center.Vertex();
      //m_normal.Vertex();
      glVertex3f(m_normal.m_x*4.0,m_normal.m_y*4.0,m_normal.m_z*4.0);
		glEnd();
	glEnable(GL_LIGHTING);
glPopMatrix();
*/
}



// modify the verts based on the dt and a flag to indicate which to tilt
void Plane::tilt(GLfloat dt,bool X,  bool Z)
{

// create a matrix for the m_xrotation and set to ident
ngl::Matrix m_xrotMatrix=1;
// create a m_zrotation matrix and set to ident
ngl::Matrix m_zrotMatrix=1;

if(X)
{
  m_xrot+=dt;
}

if (Z)
{
  m_zrot+=dt;
}

m_xrotMatrix.rotateX(m_xrot);
m_zrotMatrix.rotateZ(m_zrot);

// combine them together for the final rotation
ngl::Matrix rotMatrix=m_zrotMatrix*m_xrotMatrix;
// now modify the verts by multiplying the points with the matrix
for(int i=0; i<4; ++i)
{
  m_verts[i]=rotMatrix*m_oVerts[i];
}
m_normal=calcNormal(m_verts[3],m_verts[2],m_verts[1]);
}
